/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_DRIVER_H__
#define __CX88_DRIVER_H__

#include <pthread.h>
#include "cx88_xml_config.h"
#include "tuner_config.h"

#include <string>
#include <vector>
using namespace std;

#include "cx88_capture.h"

typedef vector<cx88_capture*> capvec;

typedef void (*cx88_capture_completion)(void*);

typedef struct
{
   cx88_capture_completion completion;
   void *completion_arg;
   unsigned int duration_s;
} cx88_capture_params;

class cx88_driver
{
   public:

      cx88_driver(
         cx88_xml_config &cx88conf,
         tuner_config &tunerconf,
         const char *devnode,
         capvec &captures);

      virtual ~cx88_driver(void) {}

      int start(const cx88_profile_t &profile, unsigned long channel, cx88_capture_params &params);
      
      int stop(void);
      
      int set_channel(const cx88_profile_t &profile, unsigned long channel);
      
      virtual int get_info(string &info) = 0;
      
   protected:
      
      cx88_xml_config &m_cx88conf;
      tuner_config &m_tunerconf;
      const char *m_devnode;
      volatile bool m_stop;

      int start_captures(void);      
      int submit_buffer(void *buffer, size_t num_bytes, size_t packet_size = 0);
      void stop_captures(void);
      
      virtual int capture(const cx88_profile_t &profile, unsigned long channel, cx88_capture_params &params) = 0;
      virtual int change_channel(const cx88_profile_t &profile, unsigned long channel) = 0;
      
   private:
      
      pthread_t m_thread;
      capvec &m_captures;
      enum
      {
         CX88_DRIVER_STOPPED,
         CX88_DRIVER_PAUSED,
         CX88_DRIVER_RUNNING
      } m_state;
      cx88_profile_t m_profile;
      unsigned long m_channel;
      cx88_capture_params m_params;
      
      static void *thread_routine(void *arg);
};

#endif
