/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_MPEG_DRIVER_FACTORY_H__
#define __CX88_MPEG_DRIVER_FACTORY_H__

#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include "pchdtv_hd3000.h"
#include "pchdtv_hd5500.h"

namespace cx88_mpeg_driver_factory
{
   
   static cx88_mpeg_driver *create(
      cx88_xml_config &cx88conf,
      tuner_config &tunerconf,
      const char *devnode,
      capvec &captures,
      int &error)
   {
      int fd = open(devnode, O_RDWR);
      if (fd < 0)
      {
         return NULL;  
      }
      uint32_t id;
      error = ioctl(fd, CX88_MPEG_CARD_ID, &id);
      close(fd);
      if (error)
      {
         return NULL;
      }
      switch (id)
      {
         case 0x70633000:
            return (new pchdtv_hd3000(cx88conf, tunerconf, devnode, captures, error));
         case 0x70635500:
            return (new pchdtv_hd5500(cx88conf, tunerconf, devnode, captures, error));
      }
      CX88ERR << "Unrecognized card ID: 0x" << hex << id << endl;
      return NULL;
   }
   
};

#endif
