/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <sys/errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "cx88_udp_capture.h"

#define UDP_MTU 1500

cx88_udp_capture::cx88_udp_capture(const char *addr, unsigned short port, int &error)
   : cx88_capture(error),
     m_socket(-1)
{
   struct in_addr iaddr;
   struct sockaddr_in dest;
   if (error)
   {
      return;
   }
   if (inet_aton(addr, &iaddr) == 0)
   {
      error = EINVAL;
      return;
   }
   if ((m_socket = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
   {
      error = errno;
      return;
   }
   socklen_t addrsize = sizeof(dest);
   if (getsockname(m_socket, (struct sockaddr *)(&dest), &addrsize) == -1)
   {
      error = errno;
      return;
   }
   dest.sin_port = ntohs(port);
   dest.sin_addr = iaddr;
   if (connect(m_socket, (struct sockaddr *)(&dest), addrsize) == -1)
   {
      error = errno;
      return;
   }
}

cx88_udp_capture::~cx88_udp_capture(void)
{
   stop();
   if (m_socket >= 0)
   {
      close(m_socket);
   }
}

void cx88_udp_capture::process_buffer(void *data, size_t num_bytes, size_t packet_size)
{
   char *bufptr = reinterpret_cast<char*>(data);
   size_t count = 0;
   ssize_t writecount;
   ssize_t msglen = UDP_MTU;
   if (packet_size != 0)
   {
      msglen = (UDP_MTU / packet_size) * packet_size;
   }
   while (count < num_bytes)
   {
      writecount = num_bytes - count;
      if (writecount > msglen)
      {
         writecount = msglen;
      }
      writecount = write(m_socket, bufptr, writecount);
      if (writecount > 0)
      {
         bufptr += writecount;
         count += writecount;
      }
      if (writecount < 0)
      {
         break;
      }
   }
}
