/*-
 * Copyright 2007 Jason Harmening
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef __CX88_XML_CONFIG_H__
#define __CX88_XML_CONFIG_H__

#ifdef _DIAGNOSTIC
#define DIAGNOSTIC(stmt) stmt;
#else
#define DIAGNOSTIC(stmt)
#endif

#define CX88ERR (cerr << "[cx88] ")
#define CX88LOG (clog << "[cx88] ")
#include <ezxml.h>

#define CX88_XML_PROFILE_NAME "profile"
#define CX88_XML_NAME_NAME "name"
#define CX88_XML_MODULATION_NAME "modulation"
#define CX88_XML_RANGE_NAME "range"
#define CX88_XML_BANDWIDTH_NAME "bandwidth"
#define CX88_XML_START_NAME "start"
#define CX88_XML_END_NAME "end"
#define CX88_XML_FREQ_NAME "freq"

typedef ezxml_t cx88_profile_t;
typedef struct 
{
   ezxml_t elem;
   unsigned long start;
   unsigned long end;
} cx88_range_t;

class cx88_xml_config
{
   public:
      
      cx88_xml_config(const char *filename, int &error);
      
      virtual ~cx88_xml_config(void);
            
      virtual int get_profile(const char *name, cx88_profile_t &profile);
      
      virtual int get_range(const cx88_profile_t &profile, unsigned long channel, cx88_range_t &range);
      
      virtual const char *get_property(const cx88_range_t &range, const char *property);
      
   private:
      
      ezxml_t m_file;
   
};

#endif
